import logging, simplejson, redis
from snmp import get_variable

#logging.basicConfig(level=logging.DEBUG)
def load_data(data):
    return simplejson.loads(data)


def cpu_usage_snmp(ctxt):
    """
    ctxt["service"].data["host"]
    ctxt["service"].data["port"]
    ctxt["service"].data["snmp_read_comunity"]
    ctxt["service"].data["snmp_mib"]
    ctxt["service"].data["snmp_variable"]    
    """
    logging.debug("[metric: cpu_usage_snmp] Entering  ...")

    logging.debug("[metric: cpu_usage_snmp] metric ended")
    return 60
    
    
def redis_list_length(ctxt):
    """
    ctxt["service"].data["redis_host"]
    ctxt["service"].data["redis_port"]
    ctxt["service"].data["redis_db"]
    ctxt["service"].data["redis_list_name"]
    """
    logging.debug("[metric: redis_list_length] Entering  ...")
    data = load_data(ctxt["service"].data)    
    
    try:
        r = redis.StrictRedis(
            host=data["redis_host"], 
            port=data["redis_port"], 
            db=data["redis_db"])
        list_length = r.llen(data["redis_list_name"])
    except Exception, er:
        logging.error("[metric: redis_list_length] Exception msg: %s" % er)
        return "[metric: redis_list_length] ERROR trying to rerieve the metric value"
    logging.debug("[metric: redis_list_length] current length: %d" % list_length)
    logging.debug("[metric: redis_list_length] metric ended")
    return list_length


def http_response_time(ctxt):
    pass


mappings = [
    cpu_usage_snmp,
    redis_list_length,
    http_response_time,
    ]


import os
import sys
import time
import uuid
import logging

'''
        @author: msune,lbergesio,cbermudo,omoya,CarolinaFernandez
	@organization: i2CAT, OFELIA FP7

	Django backend driver
	Implementes django model-based backend
'''
from .RuleModel import PolicyRuleModel
from .RuleTableModel import PolicyRuleTableModel
from pypelib.parsing.ParseEngine import ParseEngine
from pypelib.Rule import Rule
from pypelib.RuleTable import*
from pypelib.resolver.Resolver import Resolver
from pypelib.utils.Logger import Logger
from django.core.exceptions import ObjectDoesNotExist
from django.core.exceptions import MultipleObjectsReturned
from pypelib.utils.Exceptions import *


#XXX: Django is required to run this driver
class Django():

	logger = Logger.getLogger()
	#Driver attributes
	_types = ["Rule","RuleTable"]

	def __init__(self):
		raise Exception("Static class cannot be instanciated")

	'''
	Stores object
	@params:
		parser - parser to be used
	@returns persisted instance
	'''
	@staticmethod
	def save(obj, parser):

		if parser == None:
			parser = obj._parser
		try:
			ruleTable = PolicyRuleTableModel.objects.get(name=obj.name)
			ruleTable.type = obj._policy
			ruleTable.defaultParser = obj._parser
			ruleTable.defaultPersistence = obj._persistenceBackend
			ruleTable.defaultPersistenceFlag = obj._persist
			ruleTable.save()
		except:
			ruleTable = PolicyRuleTableModel(name = obj.name, uuid = obj.uuid, type = obj._policy, defaultParser = parser, defaultPersistence = obj._persistenceBackend, defaultPersistenceFlag = obj._persist)

			ruleTable.save()


		#flag to know if save comes from addRule or removeRule()
		addRule = 0
		uuidInRuleSet=list()
		for rule in obj._ruleSet:

			#need to removeRule later
			uuidInRuleSet.append(rule.rule.getUUID())
			#lbergesio: Rule is saved parsed, as a string
			#Possibly if Rule exists (try) it is not required to update its fields.==>
			#==> IT IS NEEDED FOR THE RuleTable.moveRule()
			try:
				ruleModel = PolicyRuleModel.objects.get(RuleUUID = rule.rule.getUUID())
			except:
				# If #Rules == #Models then it is the PolicyRuleTableModel being
				# deleted, not the PolicyRuleModel (last is the normal case)
				if len(obj._ruleSet) != PolicyRuleModel.objects.all().count():
				    addRule = 1
				    ruleModel = PolicyRuleModel(RuleUUID = rule.rule.getUUID())

			ruleModel.RuleTableName = obj.name
			ruleModel.Rule = ParseEngine.craftRule(rule.rule, obj._parser)
			ruleModel.RuleIsEnabled = rule.enabled
			ruleModel.RulePosition = obj._ruleSet.index(rule)
			ruleModel.save()
		#if the save() comes from a removeRule, check which one was removed and delete
		if not addRule:
			for ruleModel in PolicyRuleModel.objects.filter(RuleTableName = obj.name):
				if ruleModel.RuleUUID not in uuidInRuleSet:
					ruleModel.delete()
					break

	@staticmethod
	def load(tableName, mappings, parser):
		Django.logger.info('Django.load')
		try:
			Table =  PolicyRuleTableModel.objects.get(name = tableName)
		# Translation of exceptions: Django -> PyPElib
		except ObjectDoesNotExist:
			raise ZeroPolicyObjectsReturned("[Django Driver] There is no table with name: " + tableName)
		except MultipleObjectsReturned:
			raise MultiplePolicyObjectsReturned("[Django Driver] There are multiple tables with name: " + tableName)
		except Exception as e:
			raise Exception("[Django Driver] Some error occurred when trying to fetch table table with name: " + tableName + ". Exception: " + str(e))
		print "type de table %s" % Table.type
		ruleTable = RuleTable(Table.name,mappings,Table.defaultParser, Table.defaultPersistence,False, eval(Table.type), Table.uuid)
		ruleTable._ruleSet = Django.loadRuleSet(Table.uuid)
		ruleTable._persist = Table.defaultPersistenceFlag
		ruleTable._mappings = mappings
		ruleTable._resolver = Resolver(mappings)
		return ruleTable

	'''
	Retrieves every PolicyRuleTable object by name.
	This method should be seldom used.
	'''
	@staticmethod
	def loadAll(tableName):
		Django.logger.debug('Loading RuleTable set...')
		try:
			return PolicyRuleTableModel.objects.filter(name = tableName).order_by('id')
		except:
			Django.logger.warning('[Django Driver] Could not retrieve any PolicyRuleTable object by the name: %s' % tableName)
			return list()

        '''
        Deletes a PolicyRuleTable object for a given ID.
        This method should be seldom used.
        '''
        @staticmethod
        def delete(tableID):
                Django.logger.debug('Deleting RuleTable with id = %s...' % tableID)
                try:
                        table = PolicyRuleTableModel.objects.get(id = tableID)
			# If this is the latest PolicyRuleTable, delete its associated PolicyRule's
			if PolicyRuleTableModel.objects.filter(name=table.name).count() == 1:
				PolicyRuleModel.objects.filter(RuleTableName=table.name).delete()
			table.delete()
                except Exception as e:
                        Django.logger.warning('[Django Driver] Could not delete the PolicyRuleTable object with ID: %s. Exception: %s' % (tableID,str(e)))

	@staticmethod
	def loadRuleSet(table_uuid):
		Django.logger.debug('Loading Rule set...')
		try:
			ruleTable = PolicyRuleTableModel.objects.get(uuid = table_uuid)
		except:
			return list()
		rules = PolicyRuleModel.objects.filter(RuleTableName = ruleTable.name).order_by('RulePosition')
		ruleSet = []
		#The rules sort by priority
		for rule in rules:
			ruleObj= ParseEngine.parseRule(rule.Rule)
			ruleObj._uuid = rule.RuleUUID
			ruleEntry = RuleEntry(ruleObj,rule.RuleIsEnabled)
			ruleSet.append(ruleEntry)
		return ruleSet



import os
import sys
import time

'''
        @author: msune,omoya,CarolinaFernandez
	@@organization: i2CAT, OFELIA FP7

	Persistence engine
	Implementes driver-based persistence backend selection
'''


class PersistenceEngine():

	#Default Class Attributes
	_defaultParser = "RegexParser"
        _defaultPersistence = "Django"

	#Drivers
	_drivers = ["Django","RAWFile"]

	#Fill with appropiate path
	PATH_TO_DRIVERS="backends"

	def __init__(self):
		raise Exception("Static class cannot be instanciated")

	@staticmethod
	def _getDriver(driverName):
		print "driver name: %s" %driverName
		if driverName == "Django":
			PATH = PersistenceEngine.PATH_TO_DRIVERS + '.django.Django'
			try:
				exec('from ' + PATH + ' import Django')
				return Django
			except:
				raise Exception(driverName + ' persistence driver not found in ' + PersistenceEngine.PATH_TO_DRIVERS)
		elif driverName == "RAWFile":
			PATH = PersistenceEngine.PATH_TO_DRIVERS + '.rawfile.RAWFile'
			try:
				exec('from ' + PATH + ' import RAWFile')
				return RAWFile
			except:
				raise Exception(driverName + ' persistence driver not found in ' + PersistenceEngine.PATH_TO_DRIVERS)
		else:
			raise Exception(driverName + ' not supported')

	@staticmethod
	def save(obj, pBackend, parser=None, **kwargs):
		return PersistenceEngine._getDriver(pBackend).save(obj, parser, **kwargs)

	@staticmethod
	def load(tableName, pBackend, resolverMappings, parser=None, **kwargs):
		return PersistenceEngine._getDriver(pBackend).load(tableName, resolverMappings, parser, **kwargs)

        '''
        Retrieves every Driver's PolicyRuleTable object for a given name.
        This method should be seldom used.
        '''
	@staticmethod
	def loadAll(tableName, pBackend):
		return PersistenceEngine._getDriver(pBackend).loadAll(tableName)

        '''
        Deletes a Driver's PolicyRuleTable object for a given ID.
        This method should be seldom used.
        '''
        @staticmethod
        def delete(tableID, pBackend):
                return PersistenceEngine._getDriver(pBackend).delete(tableID)




import os
import sys
import time

from threading import Thread, Lock

'''
        @author: msune
	@organization: i2CAT, OFELIA FP7

	Resolver class
	Attemps to resolve keywords agains values/actions through mappings
'''

#Resolvers dictionary
_resolvers = {}

'''Resolver class'''
class Resolver():

	#Class attributes
	_mappings = None
	_mutex = None
	#Constructor
	def __init__(self,mappings):
		self._mutex=Lock()
		self._mappings =mappings
		pass

	#set mappings
	#@mappings: dictionary containing name <-> mapping (object...)
	def setMappings(self,mappings):
		with self._mutex:
			self._mappings = mappings

	#Get or generate the resolver
	@staticmethod
	def getOrGenerateResolver(theId,mappings=None):
		if theId in _resolvers:
			return _resolvers[theId]

		if mappings == None:
			raise Exception("Could not find the resolver with id:"+theId)

		instance = Resolver(mappings)
		#Save instance
		_resolvers[theId] = instance

		return instance

	#Try to parse as a number
	def _getNumericValue(self,string):
		try:
			#Try to parse integer
			return int(string)
		except:
			#Try a floating point
			try:
				return float(string)
			except:
				return string

	#Resolve a key
	def resolve(self, key, metaObj):
		with self._mutex:
			#print "mappings: %s\n mmmmmmmmmmmmmmmmmmmmmmmm" % self._mappings
			if not (isinstance(key,str) or  isinstance(key,unicode)):
				raise Exception("Only string keys are able to be resolved")

			if key not in self._mappings:
				#print "[DEBUG] "+str(self._getNumericValue(key))
				return self._getNumericValue(key)

			if isinstance(self._mappings[key],str):
				#print "aaaaaaaaaaaaaaaaaa[DEBUG] resolved"+str(eval(self._mappings[key]))
				return eval(self._mappings[key])
			else:
				#print "[DEBUG] Action: %s" % self._mappings[key]
				return self._mappings[key](metaObj)


#resolver = Resolver.getOrGenerateResolver("hola",{"test":"metaObj","test2":2})
#metaObj = 3
#print "Value:"+str(resolver.resolve("test",metaObj))
#print "Value:"+str(resolver.resolve("test2",metaObj))



from django.contrib import admin
from models import *

class ScaleActionAdmin(admin.ModelAdmin):
    pass
admin.site.register(ScaleAction, ScaleActionAdmin)

class ThresholdAdmin(admin.ModelAdmin):
    pass
admin.site.register(Threshold, ThresholdAdmin)

class MonitoredServiceAdmin(admin.ModelAdmin):
    pass
admin.site.register(MonitoredService, MonitoredServiceAdmin)

class AlarmIndicatorAdmin(admin.ModelAdmin):
    pass
admin.site.register(AlarmIndicator, AlarmIndicatorAdmin)




from pysnmp.entity.rfc3413.oneliner import cmdgen

class SNMPError(Exception): pass


def get_variable(udp_host, udp_port, community, mib, variable, default=None):
    cmdGen = cmdgen.CommandGenerator()
    
    errorIndication, errorStatus, errorIndex, varBinds = cmdGen.getCmd(
        cmdgen.CommunityData(community),
        cmdgen.UdpTransportTarget((udp_host, udp_port)),
        cmdgen.MibVariable(mib, variable, default),
        lookupNames=True, lookupValues=True
    )
    
    # Check for errors and print out results
    if errorIndication:
        raise SNMPError(errorIndication)
    if errorStatus:
        raise SNMPError(errorStatus)

    return [(name.prettyPrint(), val.prettyPrint()) for name, val in varBinds]



'''
        @author: CarolinaFernandez
        @organization: i2CAT, OFELIA FP7

        PolicyEngine Exceptions class
        Encapsulates custom exceptions 
'''

class ZeroPolicyObjectsReturned(Exception):
    pass

class MultiplePolicyObjectsReturned(Exception):
    pass



import os
import sys
import time
import uuid

'''
        @author: msune,lbergesio,cbermudo,omoya
	@organization: i2CAT, OFELIA FP7

	ParseEngine	
	Implementes driver-based parsing mechanism for rules
'''


class ParseEngine():

	#Drivers
	_drivers = ["RegexParser"]
	_defaultDriver = 'RegexParser'


	#Fill with appropiate path
	PATH_TO_DRIVERS="drivers"
	
	def __init__(self):
		raise Exception("Static class cannot be instanciated")	

	@staticmethod
	def _getDriver(driverName=None):
		if driverName == None or driverName not in ParseEngine._drivers:
			#TODO: Try all drivers?
			raise Exception("Cannot find parser")
	
		elif driverName == "RegexParser":

			path = '.' + ParseEngine.PATH_TO_DRIVERS + '.RegexParser'
			exec('from ' + path + ' import RegexParser') 
			#from ParseEngine.PATH_TO_DRIVERS.RegexParser import RegexParser
			return RegexParser
				
	@staticmethod
	def parseRule(string, driverName= _defaultDriver):
		return ParseEngine._getDriver(driverName).parseRule(string)

	@staticmethod
	def craftRule(rule, driverName=None):
		return ParseEngine._getDriver(driverName).craftRule(rule)

# This calls should not be here. Instead, use parseRule in the persistence drivers if required.
#	@staticmethod
#	def parseCondition(stringCond, driverName=None):
#		return ParseEngine._getDriver(driverName)._parseCondition(stringCond)
#	
#	@staticmethod
#	def craftCondition(condition, driverName=None):
#		return ParseEngine._getDriver(driverName).craftCondition(condition)





import os
import sys
import time

sys.path.append("../../") 
import re
from pyparsing import nestedExpr

'''
        @author: msune,cbermudo,omoya
	@organization: i2CAT, OFELIA FP7

	RegexParser class
'''

from pypelib.Rule import *
from pypelib.Condition import *



'''RegexParser class'''
class RegexParser():

	_rValues=["accept","deny"] #Must be lowercase

	def __init__(self):
		raise Exception("Static class cannot be instanciated")	

	#Recover group by name from a matched regex
	@staticmethod
	def _getGroupByName(match,name):
		try:
			return match.group(name)
		except:
			return None

	#Try to parse as a number
	@staticmethod
	def _getNumericValue(string):
		try:
			#Try to parse integer
			return int(string)	
		except:
			#Try a floating point
			try:
				return float(string)	
			except:
				return string
	
	@staticmethod
	def _stripExtraParenthesis(s):
		
		try:
			if s.count("(") != s.count(")"):
				raise Exception("Cannot parse condition")

			toStrip=0
			el = nestedExpr('(', ')').searchString(s).asList()
			
			if len(el) >1:
				raise Exception("non-strippable")

			def _countExtraParenthesis(el):
				if len(el) == 1 and isinstance(el[0],list):
					return 1+_countExtraParenthesis(el[0])
				return 0	
			toStrip+=_countExtraParenthesis(el[0])	

			for it in range(0,toStrip):
				match = re.match(r'[\s]*\((?P<inner>.+)\)[\s]*',s)
				s = match.group("inner")
			
		except Exception as e:
			pass
		return s 
	@staticmethod
	def _parseComplexCond(string):
                #In case of complex condition it will always be, as it is stripped, (something) 
                #re has some limitations on this
		
		string = string.replace("not(","not (")
                match = re.match(r'[\s]*(not[\s])?[\s]*\((?P<left>.+)\)[\s]*(?P<operand>&&|\|\|)[\s]*\((?P<right>.+)\)[\s]*\Z', string,re.IGNORECASE)
                neg=""
                if match:
                        neg=match.group(1) != None
                        counter = 0
                        #Complex cond
                        for iterator in range(0,len(string)-1):
                                if string[iterator] == '(':
                                        counter+=1
                                elif string[iterator] == ')':
                                        counter-=1
                                if counter== 0 and  ( string[iterator] == '&' and string[iterator+1] == '&' ):
                                        if neg:
                                                return "&&", string[0:iterator].replace("not","",1), string[iterator+2:len(string)],neg
                                        else:
                                                return "&&", string[0:iterator], string[iterator+2:len(string)],neg
#                                       return "&&", string[(0+3*int(neg)):iterator], string[iterator+2:len(string)],neg
                                elif counter == 0 and (  string[iterator] == '|' and string[iterator+1] == '|' ):
                                        if neg:
                                                return "||", string[0:iterator].replace("not","",1), string[iterator+2:len(string)],neg
                                        else:
                                                return "||", string[0:iterator], string[iterator+2:len(string)],neg
#                                       return "||", string[(0+3*int(neg)):iterator], string[iterator+2:len(string)],neg

                return None,None,None,None
 
	@staticmethod
	def _parseCondition(string):
		#first strip extra parenthesis if any
		conditionString = RegexParser._stripExtraParenthesis(string)
		#Detect complex condition (Boolean) 
		op,leftOP,rightOP,neg = RegexParser._parseComplexCond(conditionString)
		
		if op:
			#print "---->Value neg:"+str(neg)+ " left: "+leftOP+"right: "+rightOP+" op:"+op
			return Condition(RegexParser._parseCondition(leftOP),RegexParser._parseCondition(rightOP),op,neg)
		else:
			#Simple conditions
			match0 = re.match(r'[\s]*(not[\s])?[\s]*([^()\s]+)[\s]*(>=|<=)[\s]*([^()\s]+)[\s]*', conditionString,re.IGNORECASE)
			if match0:
                        	return Condition(RegexParser._getNumericValue(match0.group(2)),match0.group(4),RegexParser._getNumericValue(match0.group(3)),match0.group(1) != None)
			match = re.match(r'[\s]*(not[\s])?[\s]*([^()\s]+)[\s]*(=|!=|>|<)[\s]*([^()\s]+)[\s]*', conditionString,re.IGNORECASE)
			if match:
				return Condition(RegexParser._getNumericValue(match.group(2)),match.group(4),RegexParser._getNumericValue(match.group(3)).replace(" ",""),match.group(1) != None)	
			else:
				#Ranges and collections
				match = re.match(r'[\s]*(not[\s])?[\s]*(.+)[\s]+(in|not[\s]+in)[\s]+(collection|range)[\s]*(\{(.+)\}|\[(.+)\])[\s]*', conditionString,re.IGNORECASE)
				if match:
					negate = None
					if re.match(r'[\s]*\[(.*)]',match.group(5)):
						operator="[]"
						if match.group(4).lower() == "collection":
							raise Exception("Error while parsing Rule(Condition). Unknown operator/collection")
					else:
						operator="{}"
					negate= match.group(1)!=None or match.group(3).lower().find("not") != -1
						
					if match.group(4).lower() == "collection":
						#is collection
						submatch = re.match(r'[\s]*\{[\s]*(.*)[\s]*\}',match.group(5))
						if not submatch:
							raise Exception("Error while parsing Rule(Condition).")
						strings = submatch.group(1).split(",")
						
						if not strings:
							raise Exception("Error while parsing Rule(Condition). Substrings")
						
						
						return Condition(match.group(2),Collection(strings),"in", negate )
					else:
						#is range
						submatch = re.match(r'[\s]*(\[|\{)[\s]*(.*)[\s]*,[\s]*(.*)[\s]*(\]|\})[\s]*',match.group(5))
						if not submatch:
							raise Exception("Error while parsing Rule(Condition).")
						
					
						return Condition(match.group(2),Range(submatch.group(2),submatch.group(3)),operator, negate)
		raise Exception("Error while parsing Rule(Condition)")
			
	@staticmethod
	def parseRule(toParse):
		
		#Extracting basics of the rule	
		#match = re.match(r'[\s]*if[\s]+(?P<condition>.+)[\s]+then[\s]+(?P<rValue>\w+)[\s]+(?P<term>nonterminal)?[\s]*(?P<do>do)?[\s]*(?P<action>[^#\s]+)[\s]([\s]*denyMessage([\s])*(?P<errorMsg>[^#]+))?([\s]*#([\s]*)(?P<comment>.+))?[\s]*', toParse,re.IGNORECASE)
		match = re.match(r'[\s]*if[\s]+(?P<condition>.+)[\s]+then[\s]+(?P<rValue>\w+)[\s]*(?P<term>nonterminal)?[\s]*(do[\s]+(?P<action>[^#\s]+))?[\s]*(denyMessage[\s]+(?P<errorMsg>[^#]+))?[\s]*(#[\s]*(?P<comment>.+))?[\s]*', toParse,re.IGNORECASE)
		
		if not match:
			raise Exception("Error while parsing Rule.") 
	
		#Generate Rule
		if match.group("rValue").lower() not in RegexParser._rValues:
			raise Exception("Error while parsing Rule. Unknown Rule value")
		
		cond = RegexParser._parseCondition(match.group("condition"))
		
		description=RegexParser._getGroupByName(match,"comment")
		term=RegexParser._getGroupByName(match,"term")
		error=RegexParser._getGroupByName(match,"errorMsg")
	
		if match.group("rValue").lower() == "accept":
			if  term and term.lower() == "nonterminal":
				rType = Rule.POSITIVE_NONTERMINAL
			else:
				rType = Rule.POSITIVE_TERMINAL
		elif match.group("rValue").lower() == "deny":
			if  term and term.lower() == "nonterminal":
				rType = Rule.NEGATIVE_NONTERMINAL
			else:
				rType = Rule.NEGATIVE_TERMINAL
		
		else:
			raise Exception("Unknown return value")
		return Rule(cond,
			description,
			error,
			rType,
			RegexParser._getGroupByName(match,"action")	
			)	



	#Crafter must be independent of the Rule representation via dump()
	@staticmethod
	def _craftCondition(cond):
		string = ""
		operator=""

		if cond.getNegate():
			string+="not "
		
		if isinstance(cond.getLeftOperand(),Condition):
			#Complex condition (boolean)
			left = RegexParser._craftCondition(cond.getLeftOperand())
			right = RegexParser._craftCondition(cond.getRightOperand())
			
			string += "(%s) %s (%s)"%(left,cond.getOperator(),right)
			return string 
		else:
			#Simple condition
			if cond.getOperator() == "[]" or cond.getOperator() == "{}":
				#Range
				if not cond.getOperator() == "[]":
					string+="%s %s {%s}"%(cond.getLeftOperand(),"in range",cond.getRightOperand())
				else:
					string+="%s %s [%s]"%(cond.getLeftOperand(),"in range",cond.getRightOperand())		
						
				return string 
			elif cond.getOperator() == "in":
				#Collection	
				string+="%s %s {"%(cond.getLeftOperand(),"in collection")
				it=0
				for item in cond.getRightOperand():
					string +=item
					if it+1 != len(cond.getRightOperand()):
						 string+=","
					it+=1
				string +="}"
				return string 
			else:
				string+="%s %s %s"%(cond.getLeftOperand(),cond.getOperator(),cond.getRightOperand())		
				return string 

	@staticmethod
	def craftRule(rule):
		string  = "if %s then " %(RegexParser._craftCondition(rule.getCondition()))
		#rValue
		if rule.getType()["value"]:
			string+="accept "
		else:
			string+="deny "
		#terminal
		if not rule.getType()["terminal"]:
			string+="nonterminal "	
		#Match action
		if rule.getMatchAction():
			string+="do %s "%rule.getMatchAction()
		#ErrorMsg action
		if rule.getErrorMsg():
			string+="denyMessage %s "%rule.getErrorMsg().rstrip()
		#Description
		if rule.getDescription():
			string+="#%s "%rule.getDescription().rstrip()

		return string






from django.db import models
import simplejson
import redis
from django.conf import settings

# Create your models here.
ACTIONS = (
    ('launch_cloudforms_vmachine', 'launch_cloudforms_vmachine'),
    ('destroy_cloudforms_vmachine', 'destroy_cloudforms_vmachine'),
    )
class ScaleAction(models.Model):
    name = models.CharField(max_length=100)
    action = models.CharField(max_length=100, choices=ACTIONS)
    scale_by = models.IntegerField() #2 will scale up by two. -2 will scale down by 2
    
    def __unicode__(self):
        return u"%s" % self.name
    
    def to_dict(self):
        return {"name": self.name,
             "scale_by": self.scale_by,}            
        
    def to_redis(self, justification):
        r = redis.StrictRedis(host=settings.REDIS_HOST, port=settings.REDIS_PORT, db=settings.REDIS_DB)
        data = self.to_dict()
        data["justificaction"] = justification
        r.lpush(settings.REDIS_ACTION_LIST, simplejson.dumps(data))
        
    @staticmethod
    def from_redis(data):        
        data = simplejson.loads(data)
        return ScaleAction(name=data["name"], scale_by=["scale_by"])
   
METRICS = (
    ("cpu_usage_snmp", "cpu_usage_snmp"),
    ("redis_list_length", 'redis_list_length'),
    ("http_response_time", "http_response_time"),
    )

   
OPERANDS = (
    ('>', '>'),
    ('<','<'),
    ('=', '='),
    ('<=', '<='),
    ('>=', '>='),    
    )

class Threshold(models.Model):
    assesment = models.CharField(max_length=300)
    time_limit = models.IntegerField() #seconds the threshold must be overtaken before it becomes an alarm
    metric = models.CharField(max_length=100, choices=METRICS) #metric that is going to be monitored. It'll be a mapping.
    operand = models.CharField(max_length=10, choices=OPERANDS)
    value = models.IntegerField() # the value of the threshold
    
    def __unicode__(self):
        return u"%s" % self.assesment
    
            
class MonitoredService(models.Model):
    name = models.CharField(max_length=300)
    threshold = models.ForeignKey(Threshold)
    action = models.ForeignKey(ScaleAction)
    active = models.BooleanField(default=True)
    """
    depending on the metric that the monitored service is about, you find here 
    the needed data for that metric
    """
    data = models.TextField() 
     
    def to_pypelib(self, ):        
        return "if %s %s %s then accept" % (
                self.threshold.metric, 
                self.threshold.operand, 
                self.threshold.value)     
    
    def __unicode__(self):
        return u"%s [%s], action %s" % (self.name, self.threshold, self.action)

    
class AlarmIndicator(models.Model):
    service = models.ForeignKey(MonitoredService, related_name='alarm_indicators')
    timestamp = models.DateTimeField(auto_now_add=True)
    


import logging
from django.conf import settings
'''
        @author: msune
        @organization: i2CAT, OFELIA FP7

       	Simple Logger wrapper 
'''

logging.basicConfig(format='%(asctime)s [%(filename)s:%(lineno)d] %(levelname)s: %(message)s', datefmt='%d/%m/%Y %I:%M:%S %p', level=settings.LOG_LEVEL)


class Logger():
	@staticmethod
	def getLogger():
		#Simple wrapper. Ensures logging is always correctly configured (logging.basicConfig is executed)
		return logging.getLogger()



import os
import sys
import time
from django.db import models



'''
        @author: lbergesio,omoya,CarolinaFernandez
	@organization: i2CAT, OFELIA FP7


        Django RuleTable Model class
'''
#Django is required to run this model
class PolicyRuleTableModel(models.Model):
        class Meta:
                """RuleTable model class"""
                app_label = 'pypelibgui'
                db_table = 'pypelibgui_PolicyRuleTableModel'

        type = models.CharField(max_length = 16, default="") #terminal/non terminal
        uuid = models.CharField(max_length = 512, default="") # uuid
        name = models.TextField(max_length = 120, default="") # name
	# FIXME: set 'name' to 'unique', but that seems only possible with 'CharField'
        #name = models.CharField(max_length = 120, default="", unique=True) # name
        defaultParser = models.CharField(max_length = 64, default="", blank =True, null =True)
        defaultPersistence = models.CharField(max_length = 64, default="", blank =True, null =True)
        defaultPersistenceFlag = models.BooleanField()




import logging, redis, time
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from core.models import ScaleAction

class Command(BaseCommand):
    args = ''
    help = 'Retrieve queued actions and execute them.'



    def handle(self, *args, **options):
#        logging.basicConfig(level=logging.DEBUG)
        
        while True:
            logging.debug("[action_worker] starting loop ...")
            r = redis.StrictRedis(
                host=settings.REDIS_HOST, 
                port=settings.REDIS_PORT, 
                db=settings.REDIS_DB)
            
            while True:                
                action = r.rpop(settings.REDIS_ACTION_LIST)
                if action is None: break
                
                logging.debug("[action_worker] retrieved action: %s" % action)
                
                action = ScaleAction.from_redis(action)
                logging.debug("[action_worker] populated action: %s" % action)
            logging.debug("[action_worker] going to sleep for %ss" % settings.ACTION_WORKER_SLEEP_SECS)
            time.sleep(settings.ACTION_WORKER_SLEEP_SECS)



import os
import sys
import time

'''
        @author: msune,cbermudo,omoya
	@organization: i2CAT, OFELIA FP7

	PolicyEngine Condition class
	Encapsulates logic of a simple condition
'''

'''Supported operators'''
_collectionOperators= ["in"]
_logicalOperators = ["&&","||"]
_rangeOperators = ["[]","{}"]
_comparisonOperators = ["=","!=",">","<",">=","<="]
_operators = _comparisonOperators + _logicalOperators + _rangeOperators + _collectionOperators


'''Collection inner class'''
class Collection(list):

	def __init__(self,coll):
		self +=coll

	@staticmethod
	def isCollection(x):
		return isinstance(x,Collection)

'''Range inner class'''
class Range():
	upperLimit=None
	lowerLimit=None

	@staticmethod
	def isRange(x):
		return isinstance(x,Range)

	@staticmethod
	def isInRange(x,urange):
		if not Range.isRange(urange):
			raise Exception("Unknown data type; must be a Range instance")
		return urange.lowerLimit <= x <= urange.upperLimit

	@staticmethod
	def isInRangeStrict(x,urange):
		if not Range.isRange(urange):
			raise Exception("Unknown data type; must be a Range instance")
		return urange.lowerLimit < x < urange.upperLimit

	def __init__(self,lower,upper):
		if int(upper)<int(lower):
			raise Exception("Cannot create Range; upper limit must be equal or higher that lower limit")

		self.upperLimit = upper
		self.lowerLimit = lower


	def __str__(self):
	#	return "(%s,%s)"%(str(self.lowerLimit),str(self.upperLimit))
		return "%s,%s"%(str(self.lowerLimit),str(self.upperLimit))

'''Condition class'''
class Condition():

	#Class attributes
	_leftOperand = None
	_rightOperand = None

	_operator=None
	_negate=None

	#Getters
	def getNegate(self):
		return self._negate
	def getOperator(self):
		return self._operator
	def getLeftOperand(self):
		return self._leftOperand
	def getRightOperand(self):
		#return self._rightOperand
		return self._rightOperand

	#Constructor
	def __init__(self, left, right, operator,negate=False):

		self._leftOperand = left
		self._rightOperand = right
		self._operator = operator
		self._negate = negate

		#Make sure user is not comparing Conditions and Literals/Values
		if Condition.isCondition(self._rightOperand) is not Condition.isCondition(self._leftOperand):
			raise Exception("Nested Condition can only be logically operated with another Condition")

		#Check if one of the operands is a Condition
		if ( Condition.isCondition(self._leftOperand) ) and operator not in _logicalOperators:
			raise Exception("Cannot evaluate expression. Only logical operators can be used while using nested Conditions")

		if operator in _logicalOperators and (not Condition.isCondition(self._leftOperand) ):
			raise Exception("Cannot evaluate expression. Logical operators can only be used with nested Conditions")

		#If operator is a range Operator, check left or right is a Range instance
		if self._operator in _rangeOperators and ( not ( Range.isRange(self._rightOperand) or Range.isRange(self._leftOperand) )):
			raise Exception("Cannot operate with a range without using Range instance")

		#If operator is a collection, check right is a Collection instance
		if self._operator in _collectionOperators and not Collection.isCollection(self._rightOperand):
			raise Exception("Cannot operate with a Collection without using Collection instance")

		#Check operator
		if self._operator not in _operators:
			raise Exception("Unknown operator")

		#Check operator
		if not isinstance(self._negate,bool):
			raise Exception("Unknown negate value")

	@staticmethod
	def isCondition(x):
		return isinstance(x,Condition)

	def dump(self):
                if ( Range.isRange(self._rightOperand) ):
			negate=""
			if self._negate:
				negate="not "
			if ( self._operator == '[]' ):
				return " %s %s in range [%s] "%(negate,self._leftOperand,str(self._rightOperand))
			else:
				return " %s %s in range {%s} "%(negate,self._leftOperand,str(self._rightOperand))

		if ( Collection.isCollection(self._rightOperand) ):
			negate = ""
			if self._negate:
				negate="not "
			rightOp = "collection {"
			for item in self._rightOperand:
				rightOp = rightOp + str(item) + ","
			rightOp = rightOp[:-1] + "}"
			return " %s %s in %s "%(negate,self._leftOperand,rightOp)

		if Condition.isCondition(self._leftOperand):
			negate = ""
			if self._negate:
				negate = "not "
			return " %s (%s) %s (%s)  "%(negate,self._leftOperand.dump(),str(self._operator),self._rightOperand.dump())
		else:
			negate=""
			if self._negate:
				negate="not "
			return " %s %s %s %s   "%(negate,self._leftOperand,str(self._operator),str(self._rightOperand))


	#Evaluate condition
	def evaluate(self, metaObj,resolver):
		#Retrieve operand value
		rightValue=None
		leftValue=None

		if Condition.isCondition(self._leftOperand):
			leftValue = self._leftOperand.evaluate(metaObj,resolver)
			rightValue = self._rightOperand.evaluate(metaObj,resolver)
		else:
			#Try to retrieve value from the context (if it is not a literal)

			if Collection.isCollection(self._rightOperand):
				rightValue = self._rightOperand
			else:
				#Try to first resolve (allow string based comparison)
				try:
					rightValue = resolver.resolve(self._rightOperand,metaObj)
				except:
					rightValue =  self._rightOperand

			#Try to first resolve (allow string based comparison)
			try:
				leftValue = resolver.resolve(self._leftOperand,metaObj)
			except:
				leftValue =  self._leftOperand

		#Adjusting types for numeric comparison
		if (type(rightValue) != type(leftValue)) and (self._operator not in _rangeOperators ) and (self._operator not in _collectionOperators):
			if type(leftValue) in [int,float,complex,long] and type(rightValue)==str:
				rightValue = type(leftValue)(rightValue)
			elif type(rightValue) in [int,float,complex,long] and type(leftValue)==str:
				leftValue = type(rightValue)(leftValue)

		#for simple conditions and str Values allow only == or !=
		if (type(leftValue) == str and type(rightValue) == str) and (self._operator not in ["=","!="]):
			raise Exception("Unknown operation over string or unresolvable keyword")

		#Perform comparison and return value
		if self._operator == "=":
			return self._negate ^ (leftValue == rightValue)
		elif self._operator == "!=":
			return self._negate ^ (leftValue != rightValue)
		elif self._operator == ">":
			return self._negate ^ (leftValue > rightValue)
		elif self._operator == "<":
			return self._negate ^ (leftValue < rightValue)
		elif self._operator == ">=":
			return self._negate ^ (leftValue >= rightValue)
		elif self._operator == "<=":
			return self._negate ^ (leftValue <= rightValue)
		elif self._operator == "in":
			return self._negate ^ (leftValue in rightValue)
		elif self._operator == "[]":
			return self._negate ^ (Range.isInRange(leftValue,self._rightOperand))
		elif self._operator == "{}":
			return self._negate ^ (Range.isInRangeStrict(leftValue,self._rightOperand))
		elif self._operator == "&&":
			return self._negate ^ (leftValue and rightValue)
		elif self._operator == "||":
			return self._negate ^ (leftValue or rightValue)
		else:
			raise Exception("Unknown operator??")



import logging
from django.core.management.base import BaseCommand, CommandError
from core.rules import evaluate 

class Command(BaseCommand):
    args = ''
    help = 'Parse de date stats file to retrieve the statistics aggregation for the last N minutes.'



    def handle(self, *args, **options):
#        logging.basicConfig(level=logging.DEBUG)
        
        evaluate()
            



import os
import sys
import time
import exceptions
import uuid
import logging

'''
        @author: msune,lbergesio,omoya,CarolinaFernandez
	@organization: i2CAT, OFELIA FP7

	PolicyEngine Rule class
	Encapsulates logic of a simple Rule	 
'''

from pypelib.Condition import Condition
from pypelib.persistence.PersistenceEngine import PersistenceEngine
from pypelib.utils.Logger import Logger

class TerminalMatch(exceptions.Exception):
	value = None
	desc = None
	def __init__(self,rType,desc):
		if isinstance(rType['value'],bool):
			self.value = rType['value']
		else:
			raise Exception("Unknown rule type")
		self.desc = desc	
	def __str__(self):
		return "%s "%self.desc

class Rule():	
	
	logger = Logger.getLogger()
	#Class Attributes
	_condition = None 
	_description = None 
	_errorMsg = None
	_uuid = None #uuid.uuid4().hex
        _defaultParser = "RegexParser"
        _defaultPersistence = "Django"

	#Types of rule
	POSITIVE_TERMINAL={'value':True,'terminal':True}
	POSITIVE_NONTERMINAL={'value':True,'terminal':False}
	NEGATIVE_TERMINAL={'value':False,'terminal':True}
	NEGATIVE_NONTERMINAL={'value':False,'terminal':False}
	
	_types = [POSITIVE_TERMINAL,POSITIVE_NONTERMINAL,NEGATIVE_TERMINAL, NEGATIVE_NONTERMINAL]

	#Rule type 
	_type = None 

	#Rule match Action
	_matchAction=None


	#Getters
	def getCondition(self):
		return self._condition
	def getDescription(self):
		return self._description
	def getType(self):
		return self._type
	def getErrorMsg(self):
		return self._errorMsg
	def getMatchAction(self):
		return self._matchAction
	def getUUID(self):
		return self._uuid
	#setters
	def setUUID(self,UUID):
		self._uuid = UUID
	
	#Constructor
	def __init__(self,condition,description,errorMsg,ruleType=POSITIVE_TERMINAL,action=None,uuid=None):
		if not isinstance(condition,Condition):
			raise Exception("Object must be an instance of Condition")
		if ruleType not in self._types:
			raise Exception("Unknown rule type")
		if action == None and (ruleType == self.NEGATIVE_NONTERMINAL or ruleType == self.POSITIVE_NONTERMINAL):
			raise Exception("You cannot create non-terminal actionless rules")	

		self._condition = condition
		self._matchAction = action
		self._type = ruleType
		self._description = description
		self._errorMsg = errorMsg
		self._uuid = uuid 

	def dump(self):
		#Debug dump
		toReturn = self._condition.dump()
		
		toReturn+="=> %s "%str(self._type['value'])
		if self._matchAction != None:
			toReturn += "(%s) "%str(self._matchAction)
		if self._type['terminal']:
			toReturn += "[TERM] "
		if self._description:
			toReturn+=" #"+self._description
		return toReturn

	#Resolver is passed at evaluation time to be able to dynamically redirect actions
	def evaluate(self,metaObj,resolver):
		try:
			result = self._condition.evaluate(metaObj,resolver)
			Rule.logger.debug('Result was: %s',str(result))
		except Exception as e:
			Rule.logger.error('Error on rule: %s',self.dump())
			Rule.logger.error('Exception: %s', str(e))
			Rule.logger.error('Rule will be skiped!')
			result = False
		
		if result: 
			if self._matchAction != None:
				resolver.resolve(self._matchAction,metaObj)
			#If is terminal raise TerminalMatch
			if self._type['terminal']: 
				raise TerminalMatch(self._type,self._errorMsg)
		#return whatever	
		return

	def getConditionDump(self):
		return self.getCondition().dump()




import metrics

modules = [metrics, ]


def get_mappings():
    maps = {}
    for x in modules:
        for y in x.mappings:
            if isinstance(y, str):
                name = y.split(".")
                maps[name[len(name)-1]] = "%s" % y
                continue
            maps[y.__name__] = y
    return maps



import os
import sys
import time
import exceptions
import copy
import logging
from threading import Thread, Lock
import uuid

try:
   import cPickle as pickle
except:
   import pickle

'''
        @author: msune,lbergesio,omoya,cbermudo,CarolinaFernandez
	@organization: i2CAT, OFELIA FP7

	PolicyEngine RuleTable class
	Encapsulates logic of a simple Rule Table
'''
from pypelib.resolver.Resolver import Resolver
from pypelib.Rule import Rule,TerminalMatch
from pypelib.parsing.ParseEngine import ParseEngine
from pypelib.persistence.PersistenceEngine import PersistenceEngine
from pypelib.utils.Logger import Logger
from pypelib.utils.Exceptions import *

class RuleEntry():
	rule = None
	enabled = True

	def __init__(self,rule, enabled=True):
		self.rule = rule
		self.enabled = enabled

class RuleTable():

	logger = Logger.getLogger()
	uuid=None
	name=None
	_persist = None
	_parser = None
	_persistenceBackend = None
	_persistenceBackendParameters=None

	#Default table policy
	_policy = None
	_ruleSet = None
	_mappings = None
	_mutex = None
	_resolver = None

	#Constructor
	def __init__(self,name,resolverMappings,defaultParser, defaultPersistence, defaultPersistenceFlag, pType = False, uuid = None,**kwargs):
		if not isinstance(pType,bool):
			raise Exception("Unknown default table policy")
		self.uuid = uuid
		self.name = name
		self._mutex = Lock()
		self._policy = pType
		self._parser = defaultParser
		self._persistenceBackend = defaultPersistence
		self._persist = defaultPersistenceFlag
		self._mappings = resolverMappings
		self._ruleSet = list()
		self._persistenceBackendParameters = kwargs

		if self._persist:
			self.save(self._persistenceBackend,**kwargs)

		#Generate the resolver
		self._resolver = Resolver(resolverMappings)

        #Deep copy
        def clone(self):
                #XXX: in principle mutex is not needed since methods calling clone() are already protected
                #with self._mutex:
                cpTable = RuleTable(self.name,None,self._parser,self._persistenceBackend, False,self._policy,self.uuid, **self._persistenceBackendParameters)
                cpTable._mutex = None
                cpTable._persist = copy.deepcopy(self._persist)
                cpTable._ruleSet = copy.deepcopy(self._ruleSet)
                cpTable._resolver = None
                return cpTable


	#Determine rule position
	def _getRuleIndex(self, rule):
		for it in self._ruleSet:
			if it.rule == rule:
				return self._ruleSet.index(it)
		return None

	def getRule(self, index):
		return self._ruleSet[index].rule

	#Add, move and remove rule
	def addRule(self,string,enabled=True,pos=None,parser=None,pBackend=None,persist=True):
		if not parser:
			parser = self._parser

		rule = ParseEngine.parseRule(string, parser)
		rule.setUUID(uuid.uuid4().hex)

		with self._mutex:
			if pos > len(self._ruleSet):
				#raise Exception("Invalid position")
				self._ruleSet.append(RuleEntry(rule,enabled))
			elif pos !=None:
				self._ruleSet.insert(pos,RuleEntry(rule,enabled))
			else:
				self._ruleSet.append(RuleEntry(rule,enabled))
			if self._persist:
				self.save()

	def removeRule(self,rule=None, index=None):
		if (not rule) and (index == None):
			raise Exception("Unable to determine which rule to remove; you must specify either the rule or the index")

		with self._mutex:
			if index == None:
				index = self._getRuleIndex(rule)
				if index == None:
					raise Exception("Unable to find rule in the ruleSet")
			self._ruleSet.pop(index)
			if self._persist:
				self.save()

	def moveRule(self, newIndex, rule=None, index=None):
		if (not rule) and (index == None):
			raise Exception("Unable to determine which rule to move; you must specify either the rule or the index")

		with self._mutex:
			if index == None:
				index = self._getRuleIndex(rule)
				if index == None:
					raise Exception("Unable to find rule in the ruleSet")
			self._ruleSet.insert(newIndex, self._ruleSet.pop(index))
			if self._persist:
				self.save()

	def _modEnableRule(self, enable, rule=None,index=None):
		if (not rule) and (index == None):
			raise Exception("Unable to determine which rule to enable; you must specify either the rule or the index")

		with self._mutex:
			if index == None:
				index = self._getRuleIndex(rule)
				if index == None:
					raise Exception("Unable to find rule in the ruleSet")
			self._ruleSet[index].enabled = enable
			if self._persist:
				self.save()

	def enableRule(self,  rule=None, index=None):
		return self._modEnableRule(True,rule,index)
	def disableRule(self, rule=None, index= None):
		return self._modEnableRule(False,rule,index)

	def setPolicy(self, policy):
		if not isinstance(policy,bool):
			raise Exception("Unknown default table policy")
		with self._mutex:
			self._policy = policy
			if self._persist:
				self.save()

	def setParser(self, parser):
		with self._mutex:
			self._parser = parser
			if self._persist:
				self.save()

	def setPersistenceBackend(self, persistenceBackend):
		with self._mutex:
			self._persistenceBackend = persistenceBackend
			if self._persist:
				self.save()

	def setPersistenceFlag(self, persistenceFlag):
		with self._mutex:
			self._persist = persistenceFlag
			if self._persist:
				self.save()

	def setMappings(self, mappings):
		with self._mutex:
			self._mappings = mappings
			if self._persist:
				self.save()

	def dump(self):
		print "Table: "+self.name+" UUID: "+str(self.uuid)
		print "NUmber of rules: "+str(len(self._ruleSet))
		with self._mutex:
			i=0
			for it in self._ruleSet:
				print "[%s]:"%i +it.rule.dump()+ " Enabled: "+str(it.enabled)
				i+=1

		print "Default policy: "+str(self._policy)

	#Go through the table
	def evaluate(self,metaObj):
		#Iterate over ruleset
		with self._mutex:
			for it in self._ruleSet:
				if it.enabled:
					try:
						it.rule.evaluate(metaObj,self._resolver)
					except TerminalMatch as terminal:
						if terminal.value:
							return True
						else:
							raise terminal
			if self._policy:
				return self._policy
			else:
				raise Exception("Policy verification failed. Policy type is DENY")

	def save(self, pBackend=None,**kwargs):
		if not pBackend:
			pBackend = self._persistenceBackend
		if not kwargs:
			kwargs2 = self._persistenceBackendParameters
		else:
			kwargs2 = kwargs
		PersistenceEngine.save(self,pBackend,**kwargs2)

	#In general should not be called, use loadOrGenerate instead
	@staticmethod
	def load(name, resolverMappings, pBackend, **kwargs):
		return PersistenceEngine.load(name,pBackend,resolverMappings,**kwargs)

	@staticmethod
	def loadOrGenerate(name,resolverMappings,defaultParser, defaultPersistence, defaultPersistenceFlag, pType=False, uuid=None,**kwargs):
		try:
			return PersistenceEngine.load(name,defaultPersistence, resolverMappings, defaultParser,**kwargs)
		except ZeroPolicyObjectsReturned:
			RuleTable.logger.warning("Unable to load RuleTable, generating a new one")
			return RuleTable(name,resolverMappings,defaultParser, defaultPersistence, defaultPersistenceFlag, pType, uuid,**kwargs)
		except MultiplePolicyObjectsReturned:
			RuleTable.logger.warning("Unable to load a single RuleTable, asking the user")
			raise MultiplePolicyObjectsReturned
		except Exception as e:
			RuleTable.logger.error("Unable to load RuleTable. Exception: %s" % str(e))

        '''
        Retrieves every Engine's PolicyRuleTable object for a given name.
        This method should be seldom used.
        '''
	@staticmethod
	def loadAll(name, defaultPersistence):
		return PersistenceEngine.loadAll(name, defaultPersistence)

        '''
        Deletes a Engine's PolicyRuleTable object for a given ID.
        This method should be seldom used.
        '''
        @staticmethod
        def delete(tableID, defaultPersistence):
                return PersistenceEngine.delete(tableID, defaultPersistence)

	#Getters
        def getRuleSet(self):
                return self._ruleSet

        def getName(self):
                return self.name

        def getPolicyType(self):
                return self._policy

        def getPersistence(self):
                return self._persistenceBackend

        def getParser(self):
                return self._parser

        def getResolverMappings(self):
                return self._mappings

        def getPersistenceFlag(self):
                return self._persist



import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name = 'monscale',
    version = '0.1',
    packages = ["monscale", "monscale.core"],
    include_package_data = True,
    license = 'BSD License',
    description = 'A Django project that monitor services and acts on them',
    long_description = README,
#TODO set the project's home page
    url = 'http://blog.digitalhigh.es',
    author = 'Javier Pardo Blasco(jpardobl)',
    author_email = 'jpardo@digitalhigh.es',
    extras_require = {
        "json": "simplejson"
        },
    install_requires = (
      "Django==1.6",
      "simplejson",
      "pyparsing",
      "pytz",
      "redis",
      'pysnmp',
    ),
    
  #  test_suite='test_project.tests.runtests',
   # tests_require=("selenium", "requests"),
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



import os
import sys
import time
import copy
try:
   import cPickle as pickle
except:
   import pickle
from threading import Thread, Lock
from pypelib.resolver.Resolver import Resolver

'''
        @author: lbergesio,omoya,cbermudo
	@organization: i2CAT, OFELIA FP7

        RAWFile     
        Implementes persistence engine to a raw file for RuleTables
'''

class RAWFile():

	#XXX: lbergesio: Is it necessary to use a mutex here?
	_mutex = Lock()

	@staticmethod
	def save(obj, parser, **kwargs):
		if "fileName" not in kwargs:
			raise Exception("FileName is required")
			
		with RAWFile._mutex:
			fileObj = open(kwargs["fileName"], "wb" )

			try:
                        	cObj = obj.clone()
			except Exception,e:
				print "Could not clone original obj %s\n%s" %(str(obj),str(e))
			
			pickle.dump(cObj,fileObj)
			fileObj.close()
			
	@staticmethod
	def load(tableName, resolverMappings, parser, **kwargs):
		with RAWFile._mutex:
			if not kwargs["fileName"]:
				raise Exception("FileName is required")
		
			fileObj = open(kwargs["fileName"], "r" )
			table = pickle.load(fileObj)
			table._mutex = Lock()
			table._mappings = resolverMappings
			table._resolver = Resolver(table._mappings)
			fileObj.close()
				
			if table.name != tableName:
				raise Exception("Table name mismatch; did you specify the correct file?")
			return table





# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Deleting field 'Threshold.active'
        db.delete_column(u'core_threshold', 'active')

        # Adding field 'MonitoredService.active'
        db.add_column(u'core_monitoredservice', 'active',
                      self.gf('django.db.models.fields.BooleanField')(default=True),
                      keep_default=False)


    def backwards(self, orm):
        # Adding field 'Threshold.active'
        db.add_column(u'core_threshold', 'active',
                      self.gf('django.db.models.fields.BooleanField')(default=True),
                      keep_default=False)

        # Deleting field 'MonitoredService.active'
        db.delete_column(u'core_monitoredservice', 'active')


    models = {
        u'core.alarmindicator': {
            'Meta': {'object_name': 'AlarmIndicator'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'service': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['core.MonitoredService']"}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        u'core.monitoredservice': {
            'Meta': {'ordering': "['host']", 'object_name': 'MonitoredService'},
            'action': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['core.ScaleAction']"}),
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'host': ('django.db.models.fields.CharField', [], {'max_length': '300'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '300'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'threshold': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['core.Threshold']"}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'core.scaleaction': {
            'Meta': {'object_name': 'ScaleAction'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'scale_by': ('django.db.models.fields.IntegerField', [], {})
        },
        u'core.threshold': {
            'Meta': {'object_name': 'Threshold'},
            'assesment': ('django.db.models.fields.CharField', [], {'max_length': '300'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'metric': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'operand': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'time_limit': ('django.db.models.fields.IntegerField', [], {}),
            'value': ('django.db.models.fields.IntegerField', [], {})
        }
    }

    complete_apps = ['core']


from core.models import METRICS, Threshold, MonitoredService
from core.mappings import get_mappings
from pypelib.RuleTable import RuleTable
import logging
from django.conf import settings

import logging, datetime
from pytz import timezone
from django.conf import settings
from core.models import AlarmIndicator


def set_indicator(ctxt):
    service = ctxt["service"]
    logging.debug("[set_indicator] Entering for service: %s ..." % service)    
    
    indicator, db = AlarmIndicator.objects.get_or_create(service=service)
    if not db: indicator.save()
    
    now = datetime.datetime.utcnow().replace(tzinfo = timezone(settings.TIME_ZONE))
    logging.debug("[set_indicator] Now: %s" % now)
    logging.debug("[set_indicator] Last indicator: %s" % indicator.timestamp)
    time_diff = now - indicator.timestamp
    logging.debug("[set_indicator] Time diff: %s (%ss)" % (time_diff, time_diff.total_seconds()))
    logging.debug("[set_indicator] Seconds in threshold: %s" % service.threshold.time_limit)
    #TODO Logging
    #if time the last indicator is prior to the seconds in threshold, the action is triggered
    if time_diff.total_seconds() > float(service.threshold.time_limit):
        service.action.to_redis(str(service))
        logging.debug("[set_indicator] Action: %s is queued" % service.action)
        indicator.delete()
    logging.debug("[set_indicator] exiting for service: %s" % service)
    
def evaluate():
    mappings = get_mappings()
    for service in MonitoredService.objects.filter(active=True):
        #logging.debug("############################################### %s ###########################" % service)
        table = RuleTable("", mappings, "RegexParser",
             #rawfile,
            "RAWFile",
            None)
        table.setPolicy(False)
              
        table.addRule(service.to_pypelib())
        
        if settings.DEBUG:
            table.dump()
    
        try:
            ctxt = {"service": service}
            table.evaluate(ctxt)
            logging.info("[evaluate] service: %s has evaluate ALARM" % service)
            set_indicator(ctxt)
        except Exception:
            try:
                logging.debug("[evaluate] threshold has evaluate no alarm")
                service.alarm_indicators.get().delete()
            except AlarmIndicator.DoesNotExist: pass
            print("No ha cumplido para el servicio: %s" % service)

    


import os
import sys
import time
from django.db import models



'''
        @author: msune,omoya
	@organization: i2CAT, OFELIA FP7

	Django Rule Model class
'''
#XXX: Django is required to run this model
class PolicyRuleModel(models.Model):
	class Meta:
		"""Machine exportable class"""
                app_label = 'pypelibgui'
                db_table = 'pypelibgui_PolicyRuleModel'

	RuleUUID = models.CharField(max_length = 512, default="") # uuid
	RuleTableName = models.CharField(max_length = 512, default="", blank = True, null = True )#Table Name
	Rule = models.CharField(max_length = 2048, default="", blank =True, null =True)
	RuleIsEnabled = models.BooleanField()# Enabled or disabled Rule
	RulePosition = models.CharField(max_length = 512, default = "")#Position in RuleSet

	def __unicode__(self, ):
		return u"EL RULE: %s" % self.Rule

